Import MaxGUI.Drivers

Type TStat
	Global list:TList
	Global saved = True
	
	Function ResetList()
		list = CreateList()
	EndFunction
	
	Function AddList(name$,usrtype,u,s,k,d,a,m,t)
		TStat.saved = False
	
		Local stat:TStat = New TStat
		
		stat.name = name
		stat.usertype = usrtype
		stat.usgn = u
		stat.score = s
		stat.kills = k
		stat.deaths = d
		stat.assists = a
		stat.mvps = m
		stat.time = t
		ListAddLast(list, stat)
	EndFunction
	
	Function FindUSGN:TStat(u)
		For Local s:TStat = EachIn TStat.list
			If s.usgn = u Then Return s
		Next
	EndFunction

	Field name$
	Field usertype
	Field usgn
	Field score
	Field kills
	Field deaths
	Field assists
	Field mvps
	Field time
EndType

Function SaveStats()
	If TStat.list Then
		Local stream:TStream = WriteStream("sys/stats/userstats.dat")
		If stream Then
			TStat.saved = True
			
			If isSteamVersion()=1 Then
				WriteLine stream, "userstats steam"
			Else
				WriteLine stream, "userstats alpha"
			EndIf
			
			For Local stat:TStat = EachIn TStat.list
				WriteLine stream, stat.name
				If isSteamVersion()=1 Then
					WriteByte stream, stat.usertype
					WriteLong stream, stat.usgn
				Else
					WriteInt stream, stat.usgn
				EndIf
				WriteInt stream, stat.score
				WriteInt stream, stat.kills
				WriteInt stream, stat.deaths
				If isSteamVersion()=1 Then
					WriteInt stream, stat.assists
					WriteInt stream, stat.mvps
				EndIf
				WriteInt stream, stat.time
			Next
			
			CloseStream(stream)
			Notify("Stats saved succesfully!")
		Else
			Notify("Failed to save stats, cannot write in file", True)
		EndIf
	Else
		Notify("Failed to save stats, load stats first", True)
	EndIf
EndFunction

Function ReadStats()
	TStat.ResetList()

	If FileType("sys/stats/stats.dat") = 1 Then
		DeleteFile("sys/stats/stats.dat")
	EndIf
	
	Local stream:TStream = ReadStream("sys/stats/userstats.dat")
	If stream Then
		Local userstatsVersion:String = ReadLine(stream)
		If userstatsVersion <> "userstats alpha" And userstatsVersion <> "userstats steam" Then
			Return Null
		EndIf
		
		If userstatsVersion = "userstats alpha" Then
			SetButtonState(pre_steam_checkbox, 1)	' hack TODO: Separate version from GUI State
		Else
			SetButtonState(pre_steam_checkbox, 0)
		EndIf
	
		While Not Eof(stream)
			Local name$ = ReadLine(stream)
			Local size = StreamSize(stream) - StreamPos(stream)
			
			If size >= 20 Then
				Local usertype = 0
				Local usgn
				If isSteamVersion()=1 Then
					usertype = ReadByte(stream)
					usgn = ReadLong(stream)
				Else
					usgn = ReadInt(stream)
				EndIf
				Local score = ReadInt(stream)
				Local kills = ReadInt(stream)
				Local deaths = ReadInt(stream)
				Local assists = 0
				Local mvps = 0
				If isSteamVersion()=1 Then
					assists = ReadInt(stream)
					mvps = ReadInt(stream)
				EndIf
				Local time = ReadInt(stream)
				
				TStat.AddList(name, usertype, usgn, score, kills, deaths, assists, mvps, time)
			Else
				Exit
			EndIf
		Wend
		
		TStat.saved = True
		CloseStream(stream)
		Return True
	EndIf
EndFunction

Function Reset_e_stat_textboxes()
	SetGadgetText(e_stat_name, "")
	SetGadgetText(e_stat_usertype, "")
	SetGadgetText(e_stat_usgn, "")
	SetGadgetText(e_stat_score, "")
	SetGadgetText(e_stat_kills, "")
	SetGadgetText(e_stat_deaths, "")
	SetGadgetText(e_stat_assists, "")
	SetGadgetText(e_stat_mvps, "")
	SetGadgetText(e_stat_time, "")
EndFunction

Function Reset_stat_textboxes()
	SetGadgetText(stat_name, "")
	SetGadgetText(stat_usertype, "")
	SetGadgetText(stat_usgn, "")
	SetGadgetText(stat_score, "")
	SetGadgetText(stat_kills, "")
	SetGadgetText(stat_deaths, "")
	SetGadgetText(stat_assists, "")
	SetGadgetText(stat_mvps, "")
	SetGadgetText(stat_time, "")
EndFunction

Function LoadStatsMenu()
	If ReadStats() Then
		current_stats = Null
		Reset_stat_textboxes()
		ClearGadgetItems(stat_player_menu)
		
		For Local stat:TStat = EachIn TStat.list
			AddGadgetItem(stat_player_menu, stat.name, 0, -1, "#"+stat.usgn)
		Next
		
		EnableGadget(add_entry_button)
		EnableGadget(create_entry_button)
		EnableGadget(button_save_stats)
		EnableGadget(entry_remove_button)
	Else
		Notify("Failed to load stats", True)
	EndIf
EndFunction

Function stats_id:TStat(id)
	Local i = 0
	For Local stat:TStat = EachIn TStat.list
		If i = id Then
			Return stat
		EndIf
		i = i + 1
	Next
EndFunction

Function open_stats_id(id)
	current_stats_id = id
	current_stats = stats_id(id)
	If current_stats Then
		SetGadgetText(stat_name, current_stats.name)
		SetGadgetText(stat_usertype, current_stats.usertype)
		SetGadgetText(stat_usgn, current_stats.usgn)
		SetGadgetText(stat_score, current_stats.score)
		SetGadgetText(stat_kills, current_stats.kills)
		SetGadgetText(stat_deaths, current_stats.deaths)
		SetGadgetText(stat_assists, current_stats.assists)
		SetGadgetText(stat_mvps, current_stats.mvps)
		SetGadgetText(stat_time, current_stats.time)
	EndIf
EndFunction

Function update_stats()
	If current_stats Then		
		Local name$ = GadgetText(stat_name)
		Local usertype:Byte = 0
		Local usgn
		If isSteamVersion()=1 Then
			usertype = Byte(GadgetText(stat_usertype))
			usgn = Long(GadgetText(stat_usgn))
		Else
			usgn = Int(GadgetText(stat_usgn))
		EndIf
		Local score = Int(GadgetText(stat_score))
		Local kills = Int(GadgetText(stat_kills))
		Local deaths = Int(GadgetText(stat_deaths))
		Local assists = 0
		Local mvps = 0
		If isSteamVersion()=1 Then
			assists = Int(GadgetText(stat_assists))
			mvps = Int(GadgetText(stat_mvps))
		EndIf
		Local time = Int(GadgetText(stat_time))
		
		Local modified = False
		If name <> current_stats.name Then
			modified = True
		ElseIf usertype <> current_stats.usertype Then
			modified = True
		ElseIf usgn <> current_stats.usgn Then
			modified = True
		ElseIf score <> current_stats.score Then
			modified = True
		ElseIf kills <> current_stats.kills Then
			modified = True
		ElseIf deaths <> current_stats.deaths Then
			modified = True
		' Im adding more to your spaghetti code Starkkz!
		ElseIf assists <> current_stats.assists Then
			modified = True
		ElseIf mvps <> current_stats.mvps Then
			modified = True
		ElseIf time <> current_stats.time Then
			modified = True
		EndIf
	
		If modified Then
			current_stats.name = GadgetText(stat_name)
			current_stats.usertype = Byte(GadgetText(stat_usertype))
			If isSteamVersion()=1 Then
				current_stats.usgn = Long(GadgetText(stat_usgn))
			Else
				current_stats.usgn = Int(GadgetText(stat_usgn))
			EndIf
			current_stats.score = Int(GadgetText(stat_score))
			current_stats.kills = Int(GadgetText(stat_kills))
			current_stats.deaths = Int(GadgetText(stat_deaths))
			current_stats.assists = Int(GadgetText(stat_assists))
			current_stats.mvps = Int(GadgetText(stat_mvps))
			current_stats.time = Int(GadgetText(stat_time))
			
			ModifyGadgetItem(stat_player_menu, current_stats_id, name)
			TStat.saved = False
		EndIf
	EndIf
EndFunction

Function remove_current_entry()
	If current_stats And TStat.list Then
		ListRemove(TStat.list, current_stats)
		RemoveGadgetItem(stat_player_menu, current_stats_id)
		
		current_stats_id = Null
		current_stats = Null
		
		Reset_stat_textboxes()
	EndIf
EndFunction

Function TextFieldIntFilter(e:TEvent,context:Object)
	Select e.id
		Case EVENT_KEYCHAR
			If e.data = 8 Then Return 1
			Return Instr("0123456789", Chr(e.data))
	EndSelect
EndFunction

Function add_stat_list()
	Local name$ = GadgetText(e_stat_name)
	Local usertype:Byte = 0
	Local usgn
	If isSteamVersion()=1 Then
		usertype = Byte(GadgetText(e_stat_usertype))
		usgn = Long(GadgetText(e_stat_usgn))
	Else
		usgn = Int(GadgetText(e_stat_usgn))
	EndIf
	Local score = Int(GadgetText(e_stat_score))
	Local kills = Int(GadgetText(e_stat_kills))
	Local deaths = Int(GadgetText(e_stat_deaths))
	Local assists = 0
	Local mvps = 0
	If isSteamVersion()=1 Then
		assists = Int(GadgetText(e_stat_assists))
		mvps = Int(GadgetText(e_stat_mvps))
	EndIf
	Local time = Int(GadgetText(e_stat_time))
	
	If usgn > 0 Then
		If Len(name) > 0 Then
			If Not TStat.FindUSGN(usgn) Then
				TStat.AddList(name, usertype, usgn, score, kills, deaths, assists, mvps, time)
				AddGadgetItem(stat_player_menu, name, 0, -1, "#"+usgn)
				Return True
			Else
				Notify("This U.S.G.N ID is already in the stats list!", True)
			EndIf
		Else
			Notify("You must specify a name!", True)
		EndIf
	Else
		Notify("You must specify an usgn id!", True)
	EndIf
EndFunction

Function isSteamVersion()
	Return ButtonState(pre_steam_checkbox)=0
EndFunction

AppTitle = "Stats Editor by Starkkz v2"

Global current_stats_id
Global current_stats:TStat

Global ButtonsBaseX = 10
Global ButtonsBaseY = 5

Global EntryBaseX = 10
Global EntryBaseY = 150

Global main_window:TGadget = CreateWindow(AppTitle, DesktopWidth()/2 - 150, DesktopHeight()/2 - 200, 235, 630, Null, WINDOW_TITLEBAR)
Global button_load_stats:TGadget = CreateButton("Load Stats", ButtonsBaseX, ButtonsBaseY, 100, 25, main_window)
Global button_save_stats:TGadget = CreateButton("Save Stats", ButtonsBaseX, ButtonsBaseY+30, 100, 25, main_window)

Global create_entry_button:TGadget = CreateButton("Create Entry", ButtonsBaseX+110, ButtonsBaseY, 100, 25, main_window)
Global entry_remove_button:TGadget = CreateButton("Remove Entry", ButtonsBaseX+110, ButtonsBaseY+30, 100, 25, main_window)

Global pre_steam_checkbox:TGadget = CreateButton("pre-Steam version", ButtonsBaseX, ButtonsBaseY+60, 150, 15, main_window, BUTTON_CHECKBOX)

CreateLabel("Select entry:", EntryBaseX, EntryBaseY-60, 100, 20, main_window)
Global stat_player_menu:TGadget = CreateComboBox(EntryBaseX, EntryBaseY-40, 205, 25, main_window)

Global entry_window:TGadget = CreateWindow("Create Entry", DesktopWidth()/2 - 125, DesktopHeight()/2 - 175, 200, 475, main_window, WINDOW_TITLEBAR|WINDOW_HIDDEN)
Global add_entry_button:TGadget = CreateButton("Add Entry", 10, 420, 100, 25, entry_window)

SetGadgetAlpha(entry_window, 0.9)

DisableGadget(add_entry_button)
DisableGadget(create_entry_button)
DisableGadget(button_save_stats)
DisableGadget(entry_remove_button)

CreateLabel("Player Name", EntryBaseX, EntryBaseY, 100, 20, main_window)
CreateLabel("User type", EntryBaseX, EntryBaseY+50, 100, 20, main_window)
CreateLabel("USGN ID/Steam64 ID", EntryBaseX, EntryBaseY+100, 150, 20, main_window)
CreateLabel("Score", EntryBaseX, EntryBaseY+150, 100, 20, main_window)
CreateLabel("Kills", EntryBaseX, EntryBaseY+200, 100, 20, main_window)
CreateLabel("Deaths", EntryBaseX, EntryBaseY+250, 100, 20, main_window)
CreateLabel("Assists", EntryBaseX, EntryBaseY+300, 100, 20, main_window)
CreateLabel("MVPs", EntryBaseX, EntryBaseY+350, 280, 20, main_window)
CreateLabel("Server time", EntryBaseX, EntryBaseY+400, 100, 20, main_window)

CreateLabel("Player Name", 10, 10, 100, 20, entry_window)
CreateLabel("User Type", 10, 55, 100, 20, entry_window)
CreateLabel("USGN ID/Steam64 ID", 10, 100, 150, 20, entry_window)
CreateLabel("Score", 10, 145, 100, 20, entry_window)
CreateLabel("Kills", 10, 190, 100, 20, entry_window)
CreateLabel("Deaths", 10, 235, 100, 20, entry_window)
CreateLabel("Assists", 10, 280, 100, 20, entry_window)
CreateLabel("MVPs", 10, 325, 100, 20, entry_window)
CreateLabel("Server time", 10, 370, 100, 20, entry_window)

Global stat_name:TGadget = CreateTextField(EntryBaseX, EntryBaseY+20, 205, 25, main_window)
Global stat_usertype:TGadget = CreateTextField(EntryBaseX, EntryBaseY+70, 40, 25, main_window)
Global stat_usgn:TGadget = CreateTextField(EntryBaseX, EntryBaseY+120, 170, 25, main_window)
Global stat_score:TGadget = CreateTextField(EntryBaseX, EntryBaseY+170, 120, 25, main_window)
Global stat_kills:TGadget = CreateTextField(EntryBaseX, EntryBaseY+220, 120, 25, main_window)
Global stat_deaths:TGadget = CreateTextField(EntryBaseX, EntryBaseY+270, 120, 25, main_window)
Global stat_assists:TGadget = CreateTextField(EntryBaseX, EntryBaseY+320, 120, 25, main_window)
Global stat_mvps:TGadget = CreateTextField(EntryBaseX, EntryBaseY+370, 120, 25, main_window)
Global stat_time:TGadget = CreateTextField(EntryBaseX, EntryBaseY+420, 120, 25, main_window)

Global e_stat_name:TGadget = CreateTextField(10, 30, 170, 25, entry_window)
Global e_stat_usertype:TGadget = CreateTextField(10, 75, 40, 25, entry_window)
Global e_stat_usgn:TGadget = CreateTextField(10, 120, 170, 25, entry_window)
Global e_stat_score:TGadget = CreateTextField(10, 165, 120, 25, entry_window)
Global e_stat_kills:TGadget = CreateTextField(10, 210, 120, 25, entry_window)
Global e_stat_deaths:TGadget = CreateTextField(10, 255, 120, 25, entry_window)
Global e_stat_assists:TGadget = CreateTextField(10, 300, 120, 25, entry_window)
Global e_stat_mvps:TGadget = CreateTextField(10, 345, 120, 25, entry_window)
Global e_stat_time:TGadget = CreateTextField(10, 390, 120, 25, entry_window)

SetGadgetFilter(stat_usertype, TextFieldIntFilter)
SetGadgetFilter(stat_usgn, TextFieldIntFilter)
SetGadgetFilter(stat_score, TextFieldIntFilter)
SetGadgetFilter(stat_kills, TextFieldIntFilter)
SetGadgetFilter(stat_deaths, TextFieldIntFilter)
SetGadgetFilter(stat_assists, TextFieldIntFilter)
SetGadgetFilter(stat_mvps, TextFieldIntFilter)
SetGadgetFilter(stat_time, TextFieldIntFilter)

SetGadgetFilter(e_stat_usertype, TextFieldIntFilter)
SetGadgetFilter(e_stat_usgn, TextFieldIntFilter)
SetGadgetFilter(e_stat_score, TextFieldIntFilter)
SetGadgetFilter(e_stat_kills, TextFieldIntFilter)
SetGadgetFilter(e_stat_deaths, TextFieldIntFilter)
SetGadgetFilter(e_stat_assists, TextFieldIntFilter)
SetGadgetFilter(e_stat_mvps, TextFieldIntFilter)
SetGadgetFilter(e_stat_time, TextFieldIntFilter)

Repeat
	WaitEvent()
	Select EventID()
		Case EVENT_GADGETACTION
			Select EventSource()
				Case button_load_stats
					LoadStatsMenu()
				Case button_save_stats
					update_stats()
					SaveStats()
				Case stat_player_menu
					update_stats()
					open_stats_id(EventData())
				Case entry_remove_button
					remove_current_entry()
				Case create_entry_button
					ShowGadget(entry_window)
				Case add_entry_button
					If add_stat_list() Then
						HideGadget(entry_window)
						
						Reset_e_stat_textboxes()
					EndIf
				Case pre_steam_checkbox
					If isSteamVersion()=1 Then
						EnableGadget(stat_usertype)
						EnableGadget(stat_assists)
						EnableGadget(stat_mvps)
						EnableGadget(e_stat_usertype)
						EnableGadget(e_stat_assists)
						EnableGadget(e_stat_mvps)
					Else
						DisableGadget(stat_usertype)
						DisableGadget(stat_assists)
						DisableGadget(stat_mvps)
						DisableGadget(e_stat_usertype)
						DisableGadget(e_stat_assists)
						DisableGadget(e_stat_mvps)
					EndIf
			EndSelect
		Case EVENT_WINDOWCLOSE
			Select EventSource()
				Case main_window
					If Not TStat.saved Then
						If Proceed("Stats haven't been saved, do you wish to save them?") Then
							SaveStats()
						EndIf
					EndIf
					End
				Case entry_window
					HideGadget(entry_window)
					
					Reset_e_stat_textboxes()
			EndSelect
	EndSelect
Forever